/*	
* HYPE_Controller
*	
* Copyright (c) 2016 Brian Adam Douglas
*
* Dual licensed under the MIT and GPL licenses.
* http://en.wikipedia.org/wiki/MIT_License
* http://en.wikipedia.org/wiki/GNU_General_Public_License
*/

/**
* @class HYPE_Controller (HYPE Instance Class);
* @description The basic Controller class
*/

var HYPE_Controller = Class.extend({
/**
* Constructor
* @description HYPE_Controller combines all element property Get/Set + Symbol Instance Timeline control into one object.
* @param {Object} hypeDoc - reference to the hypeDocument
* @param {String} id - the generically, or specifically set ID of the element instance
* @param {Object} symbol - reference to the symbol that the element is an instance of
* @return null
*/
construct: function(hypeDoc, id, symbol) { 
	this.SC.construct();
	this._ID = id;
	this._SYMBOL = hypeDoc.getSymbolInstanceById(id);
	this._LABELS = new Object();
	this._HD = hypeDoc;
	this._CURRENTTIMELINE = "Main Timeline";
},


/**
* id
* @description Returns the actual ID of the element, not the variable name that we work with in the greater code structure.
* @return (String)
*/
id: function(){
  return this._ID;
},

/**
* symbol
* @description Returns the symbol that the element is an instance parent.
* @return (Object)
*/
symbol: function(){
  return this._SYMBOL;
},

/**
* addLabel
* @description Add a Label to a specific Timeline, *note all labels must be unique in any given element.
* @param {String} timelineName - name of the Timeline
* @param {String} labelName - name of the label
* @param {Interger} frameNum - the frame number associated with the label
* @param {String} onStartFrameFunc - the name of the function that fires on moving the playhead to the label (optional)
* @param {String} desc - description of the label and event, for debugging (optional)
* @return null
*/
addLabel: function(timelineName, labelName, frameNum, onStartFrameFunc, desc){
  this._LABELS[labelName]= {timeline:timelineName, frameNumber:frameNum, onStartFrame:onStartFrameFunc, description:desc};
},

/**
* getLabel
* @description Returns object associated with a Label.
* @return (Object)
*/
getLabel: function(labelName){
  return this._LABELS[labelName];
},

/**
* labelOnStartFrame
* @description Returns the function name associated with playhead moving to label
* @param {String} labelName - name of the label
* @return (String)
*/
labelOnStartFrame: function(labelName){
  return this._LABELS[labelName].onStartFrame;
},

/**
* element
* @description Returns the element associated with the element ID
* @return (Object)
*/
element: function(){
  return this._HD.getElementById(this._ID);
},

/**
* getProp
* @description Returns the specified property of an element/instance
* @param {String} propertyName - name of the property
* @return (*)
*/
getProp: function(propertyName){
  return this._HD.getElementProperty(this.element(), propertyName);
},

/**
* setProp
* @description Add a Label to a specific Timeline.
* @param {String} propertyName - name of the property being set
* @param {*} value - the value that is being set
* @param {Number} optionalDuration - see HYPE methods, duration of value change(optional)
* @param {String} optionalTimingFunctionName - see HYPE methods, this is for easing(optional)
* @return null
*/
setProp: function(propertyName, value, optionalDuration, optionalTimingFunctionName){
  this._HD.setElementProperty(this.element(), propertyName, value, optionalDuration, optionalTimingFunctionName);
},

/**
* convertFrame
* @description Convert a frame number to a time signature used for seeking playhead with HYPE's built in timeline methods
* @param {Number} frame - the frame number to seek to
* @return (Number)
*/
convertFrame: function(frame){
		if(typeof frame == 'string'){
			frame = this.getLabel(frame).frameNumber;
		}
		var seekTime = frame * 0.03333333333333;//Hype is set to 30fps, so 0.033 is the actual length of one frame
		return(seekTime);
},

/**
* height
* @description Get/Set the height of the element
* @param {Number} height - the height to set/get
* @param {Number} optionalDuration - see HYPE methods, duration of value change(optional)
* @param {String} optionalTimingFunctionName - see HYPE methods, this is for easing(optional)
* @return (Number)
*/
height: function(height, optionalDuration, optionalTimingFunctionName){
	if(height == undefined){
		return this.getProp('height');
	}
	this.setProp('height', height, optionalDuration, optionalTimingFunctionName);
},

/**
* width
* @description Get/Set the width of the element
* @param {Number} width - the width to set/get
* @param {Number} optionalDuration - see HYPE methods, duration of value change(optional)
* @param {String} optionalTimingFunctionName - see HYPE methods, this is for easing(optional)
* @return (Number)
*/
width: function(width, optionalDuration, optionalTimingFunctionName){
	if(width == undefined){
		return this.getProp('width');
	}
	this.setProp('width', width, optionalDuration, optionalTimingFunctionName);
},

/**
* x
* @description Get/Set the x position of the element
* @param {Number} x - the x(left) position to set/get
* @param {Number} optionalDuration - see HYPE methods, duration of value change(optional)
* @param {String} optionalTimingFunctionName - see HYPE methods, this is for easing(optional)
* @return (Number)
*/
x: function(x, optionalDuration, optionalTimingFunctionName){
	if(x == undefined){
		return this.getProp('left');
	}
	this.setProp('left', x, optionalDuration, optionalTimingFunctionName);
},

/**
* y
* @description Get/Set the y position of the element
* @param {Number} y - the y(top) position to set/get
* @param {Number} optionalDuration - see HYPE methods, duration of value change(optional)
* @param {String} optionalTimingFunctionName - see HYPE methods, this is for easing(optional)
* @return (Number)
*/
y: function(y, optionalDuration, optionalTimingFunctionName){
	if(y == undefined){
		return this.getProp('top');
	}
	this.setProp('top', y, optionalDuration, optionalTimingFunctionName);
},

/**
* alpha
* @description Get/Set the opacity of the element
* @param {Number} alpha - the alpha to set/get
* @param {Number} optionalDuration - see HYPE methods, duration of value change(optional)
* @param {String} optionalTimingFunctionName - see HYPE methods, this is for easing(optional)
* @return (Number)
*/
alpha: function(alpha, optionalDuration, optionalTimingFunctionName){
	if(alpha == undefined){
		return this.getProp('opacity');
	}
	this.setProp('opacity', alpha, optionalDuration, optionalTimingFunctionName);
},

/**
* rotation
* @description Get/Set the rotation of the element
* @param {Number} rotation - the rotation to set/get
* @param {Number} optionalDuration - see HYPE methods, duration of value change(optional)
* @param {String} optionalTimingFunctionName - see HYPE methods, this is for easing(optional)
* @return (Number)
*/
rotation: function(rotation, optionalDuration, optionalTimingFunctionName){
	if(rotation == undefined){
		return this.getProp('rotateZ');
	}
	this.setProp('rotateZ', rotation, optionalDuration, optionalTimingFunctionName);
},

/**
* xScale
* @description Get/Set the xScale of the element
* @param {Number} xScale - the xScale to set/get
* @param {Number} optionalDuration - see HYPE methods, duration of value change(optional)
* @param {String} optionalTimingFunctionName - see HYPE methods, this is for easing(optional)
* @return (Number)
*/
xScale: function(scale, optionalDuration, optionalTimingFunctionName){
	if(scale == undefined){
		return this.getProp('scaleX');
	}
	this.setProp('scaleX', scale, optionalDuration, optionalTimingFunctionName);
},

/**
* yScale
* @description Get/Set the yScale of the element
* @param {Number} yScale - the yScale to set/get
* @param {Number} optionalDuration - see HYPE methods, duration of value change(optional)
* @param {String} optionalTimingFunctionName - see HYPE methods, this is for easing(optional)
* @return (Number)
*/
yScale: function(scale, optionalDuration, optionalTimingFunctionName){
	if(scale == undefined){
		return this.getProp('scaleY');
	}
	this.setProp('scaleY', scale, optionalDuration, optionalTimingFunctionName);
},

/**
* start
* @description Start playing a Timeline
* @param {String} timeline - the Timeline to play
* @param {String} direction - "forward" or "reverse"
* @return null
*/
start: function(timeline, direction){
	if(direction == 'forward'){
		this._SYMBOL.startTimelineNamed(timeline, this._HD.kDirectionForward);
	}else{
		this._SYMBOL.startTimelineNamed(timeline, this._HD.kDirectionReverse);
	}
	this._CURRENTTIMELINE = timeline;
},

/**
* pause
* @description Pause the current Timeline
* @return null
*/
pause: function(){
	this._SYMBOL.pauseTimelineNamed(this._CURRENTTIMELINE);
},

/**
* resume
* @description Resume playing the current Timeline
* @param {String} direction - "forward" or "reverse"
* @param {Boolean} canRestartTimeline - see HYPE methods, if timeline ends go to the beginning and replay(optional)
* @return null
*/
resume: function(direction, canRestartTimeline){
	if(direction == 'forward'){
		this._SYMBOL.continueTimelineNamed(this._CURRENTTIMELINE, this._HD.kDirectionForward, canRestartTimeline);
	}else{
		this._SYMBOL.continueTimelineNamed(this._CURRENTTIMELINE, this._HD.kDirectionReverse, canRestartTimeline);
	}
},

/**
* seekToLabel
* @description Seek to a specific Label
* @param {String} labelName - the name of the label
* @return null
*/
seekToLabel: function(labelName){
	var labelInfo = this.getLabel(labelName);
	var timeline = labelInfo.timeline;
	this._CURRENTTIMELINE = timeline;
	var seconds = this.convertFrame(labelInfo.frameNumber);
	this._SYMBOL.goToTimeInTimelineNamed(seconds, timeline);
},

/**
* seekToFrame
* @description Seek to a specific frame in a specified Timeline
* @param {String} timeline - the name of the Timeline
* @param {Number} frameNumber - the frame number
* @return null
*/
seekToFrame: function(timeline, frameNumber){
	var seconds = this.convertFrame(frameNumber);
	this._CURRENTTIMELINE = timeline;
	this._SYMBOL.goToTimeInTimelineNamed(seconds, timeline);
},

/**
* currentFrame
* @description Get the current frame in the current timeline
* @return (Number)
*/
currentFrame: function(){
	return Math.floor(this._SYMBOL.currentTimeInTimelineNamed(this._CURRENTTIMELINE) * 30);
},

/**
* duration
* @description Get the duration for the specified timeline in terms of frames
* @param {String} timeline - the name of the Timeline
* @return (Number)
*/
duration: function(timeline){
	return Math.floor(this._SYMBOL.durationForTimelineNamed(timeline) * 30);
},

/**
* isPlaying
* @description Determine whether the specified Timeline is playing
* @param {String} timeline - the name of the Timeline
* @return (Boolean)
*/
isPlaying: function(timeline){
	return this._SYMBOL.isPlayingTimelineNamed(timeline);
},

/**
* onStartFrame
* @description This event fires the function that is internal to the HYPE Document that is associate with the onStartFrame. The functiosn are kept internal to the 
* document for purposes of encapsulation of this class
* @param {String} labelName - the name of the label that had been reached
* @return null
*/
onStartFrame: function(labelName){
	var func = this.getLabel(labelName).onStartFrame;
	this._HD.functions()[func]();
}



 
});
